-- ##########################################################################
-- #
-- #  Licensed Materials - Property of IBM
-- #  5698-EAS
-- #
-- #  (C) Copyright IBM Corp. 2002.  All Rights Reserved.
-- #
-- #  US Government Users Restricted Rights - Use, duplication or
-- #  disclosure restricted by GSA ADP Schedule Contract with
-- #  IBM Corp.
-- #
-- #   File Information: src/com/tivoli/tec/data_warehouse/ec1/etl/sql/EC1_c10_s030_Load.db2, tec, tec_3.9.0
-- #   Revision: 1.7
-- #   Date: 6/25/04 06:40:00
-- ############################################################################

-- EC1_c10_s030_Load.db2

--       The Load script reads six staging tables populated by the Transform 
--	 script - one table for each Availability class type that the ETL
--	 should process. Preliminary error checking is done by the Availability
--	 rules that came with this warehouse enablement pack because it is more  
--	 easily done in Prolog and because, if there is an error, the rules will
--	 generate an AVAIL_Error event and not insert the values in its fact files
--	 nor generate an AVAIL_* event to be extracted by the ETL. This keeps the
--	 the rule's fact files in sync with the CDW AVAIL values. In addition, when
--	 an AVAIL_* event generates an error, for example an event of class
--	 AVAIL_Comp_Type_Relationship, then any AVAIL_Outage events of the failed
--	 component type will fail rules processing as well and be dropped. In this 
--	 way the extract will not be trying to insert outage data for component types 
--	 that don't exist.

-- 	 The Availability data is inserted into the CDW tables using a "where not 
--	 exists" clause. This insures duplicate data is not inserted, even in the Msmt 
--	 table, and an error is not generated for trying to insert duplicate data.
--	 Duplicate data in the set of extracted events that would also cause an 
--	 insertion error, are checked for, an error message is printed for those
--	 found, and finally they are removed.
--
--	 Error checking is done to ensure that data that must be defined before it
--	 can be referenced by other events already exists before it is referenced.
-- 	 For example, a component type must be defined before it's possible 
--	 measurement states are defined. If it is not already defined an error
--	 message is printed and the event is dropped from the table.
--
--	 Checks are also done to ensure that new component types have defined 
--	 measurement states in the MsmtRul table and that MGrp_Cd's have
--	 defined measurement states in the MGrpMbr table. This is done because
--	 the ITSLA ETL requires that:
--	 	1) component types used in its reports have defined measurement
--	 	   types in the MsmtRul table
--	 	2) measurement groups are defined in the MGrp and MGrpMbr tables
--	 	   that define all the unique groupings of the allowable 
--	 	   measurement states for the component types
--	 	   For example if we have the following component types and
--	 	   allowable measurement states:
--	 	   CompTypA     Degrading, Unavailable
--	 	   CompTypB     Degrading, Unavailable
--	 	   CompTypC     Unavailable
--	 	   Then we would need two measurement groups defined similar to
--	 	   the following:
--	 	   MsrGp1     Degrading, Unavailable
--	 	   MsrGp2     Unavailable
--	 If these errors are found a message is printed out but the events are not
--	 removed because they are valid but additional definitions are required. The
--	 error message details what must be done.
--
--	 Assumptions:
--	  	- Rules processing catches the following possible errors,
--		  generates AVAIL_error events and drops the event.
--		  	1) The lengths of any character slots must be less than
--		  	   or equal to the maximum length for this column in the
--		  	   warehouse. The allowable lengths are documented in the 
--		  	   enablement template. 
--		  	2) The child_comp_descr slot of class AVAIL_Comp_Relationship
--		  	   does not require a value but the rules will insert the string
--		  	   value 'NULL' if there isn't a value to make the extract 
--		  	   processing easier.  The child_comp_type_descr slot of class 
--			   AVAIL_Comp_Type_Relationship MUST have a value because the 
--			   warehouse requires a value. The descr slot of class 
--			   AVAIL_Measurement_Group_Type MUST have a value because the 
--			   warehouse requires a value. 
--		  	3) The format of the outage_start_time slot of class AVAIL_Outage
--		  	   is checked to try to ensure it is correctly formatted.
--	 
--	 TDW sql requirements for 'where not exists' clauses:
--	       1) Use 'select 1' instead of 'select *' or 'select <col_list>' because it
--		  performs better since it doesn't go search the system tables for the
--		  table's columns. Any non-column name could be used in place of '1' but
--		  '1' is less typing.
--	       2) The where clause that is part of the 'where not exists (select..... where'
--		  should only have the format 'where a.col=b.col'. Having 'where a.col <> 
--		  'literal' or 'where a.col = (select somecol from sometable)' perform very
--		  badly with DB2's optimizer and should be avoided. Where required create 
--		  temporary tables to set the values in the temporary table as needed so 
--		  these constructs are avoided in the 'where not exists' clause. The 
--		  temporary tables are needed then when columns to be inserted into the 
--		  CDW table are '(select ....)' or 'literal' instead of a column name.
--   	       3) The TWG tables that begin with 'Cur_*' are views of the tables that
--		  the name ends in. For example, Cur_Comp is a view of the Comp table.
--		  These views show only current data so searches of them take less time.
--		  The actual table, Comp in this example, has current and retired components.
--   	       4) The TWG.Msmt table may NOT have 'where not exists' used on the sql 
--		  statement to insert into that table to prevent duplicates because the  
--		  TWG.Msmt table is generally very large and the 'where not exists' clause
--		  would impact performance. Instead this work is done against a temporary
--		  table that is a copy of any rows in the Msmt table that have MSrc_Cd of 
--		  'MODEL1'. This will limit the data in the temporary table to any Msmt 
--		  data that uses this MSrc_Cd including our Availability data, but is 
--		  less than the total data stored in the Msmt table.


-- *****************************************************************************************
-- ****** START: Process the Component Type data into the CompTyp and RelnRul tables
-- *****************************************************************************************

-- *************************************************************************************
-- Two types of duplicates are checked for in this step. The first checks for 
-- duplicates that exist within the EC1.Stage_Comp_Typ_Rel table, which contains the 
-- current set of extracted events to define new component types. And the second
-- checks for duplicates of records in the EC1.Stage_Comp_Typ_Rel table that already 
-- exist in the TWG.CompTyp table. This second way is done with the 'where not exists'
-- clause on the 'insert into TWG.CompTyp' statement.
-- *************************************************************************************

-- **********************************************************************************
-- Write out an error message for any records with duplicate child_comp_type values
-- **********************************************************************************
--#EXECUTE_AT_TARGET
insert into EC1.Exception_Log
        select Current Timestamp - Current Timezone,
               'EC1_c10_ETL1_Process',
               'EC1_c10_s030_Load',
               'CompTyp',
               'checking for duplicate component type values in the EC1.Stage_Comp_Typ_Rel table before TWG.CompTyp table is updated',
               'child_comp_type_descr: ' || d1.child_comp_type_ds,
               'parent_comp_type: ' || d1.parent_comp_type,
               'child_comp_type: ' || d1.child_comp_type,
               'The child_comp_type, ' || child_comp_type || ', of class AVAIL_Comp_Type_Relationship is a duplicate component type value. No entry for this record can be entered into the CompTyp table. The first event with this component type value will be entered but this event will be discarded.', 
               'No recovery is needed. You cannot have identical component types in the TWG.CompTyp table. There were duplicate component types in this ETLs extracted events. Only the first will be inserted into the TWG.CompTyp table. All other duplicates of this component type in the extracted events will generate an error message and then be deleted.',
               '9999-01-01-00.00.00.000000'
	from EC1.Stage_Comp_Typ_Rel d1
		where d1.date_reception > (select min(d2.date_reception) from EC1.Stage_Comp_Typ_Rel d2
						where d1.child_comp_type   = d2.child_comp_type and 
               		      		      	      d1.date_reception    > d2.date_reception)
	       	       OR d1.event_hndl > (select min(d2.event_hndl) from EC1.Stage_Comp_Typ_Rel d2
						where d1.child_comp_type   = d2.child_comp_type and 
               		      		              d1.date_reception    = d2.date_reception and
               		      		              d1.event_hndl        > d2.event_hndl);
-- *******************************************************************
-- Remove any duplicate events in the newly extracted set of events
-- by removing the duplicates that arrived last.
-- *******************************************************************
--#EXECUTE_AT_TARGET
delete from EC1.Stage_Comp_Typ_Rel d1
	where d1.date_reception > (select min(d2.date_reception) from EC1.Stage_Comp_Typ_Rel d2
					where d1.child_comp_type   = d2.child_comp_type and 
               		      		      d1.date_reception    > d2.date_reception)
	      OR d1.event_hndl > (select min(d2.event_hndl) from EC1.Stage_Comp_Typ_Rel d2
					where d1.child_comp_type   = d2.child_comp_type and 
               		      		      d1.date_reception    = d2.date_reception and
               		      		      d1.event_hndl        > d2.event_hndl);

-- ***************************************************************************
-- Insert any new Component Type values into the TWG.CompTyp table
-- ***************************************************************************
-- The CompTyp_Parent_Cd is always NULL for the AVAIL component types 
-- so it is not included in the insert statement.
--#EXECUTE_AT_TARGET
insert into TWG.CompTyp
	(CompTyp_Cd, CompTyp_Nm, CompTyp_Strt_DtTm, CompTyp_End_DtTm)
	select 
	       child_comp_type                      as CompTyp_Cd, 
	       child_comp_type_ds                   as CompTyp_Nm,
	       current timestamp - current timezone as CompTyp_Strt_DtTm, 
	       '9999-01-01-00.00.00.000000'         as CompTyp_End_DtTm

	from EC1.Stage_Comp_Typ_Rel
		where not exists (select 1 from TWG.Cur_CompTyp  
					where CompTyp_Cd = child_comp_type);

-- ********************************************************************************
-- Write out an error message for any records whose parent_comp_type doesn't exist
-- ********************************************************************************
--#EXECUTE_AT_TARGET
insert into EC1.Exception_Log
        select Current Timestamp - Current Timezone,
               'EC1_c10_ETL1_Process',
               'EC1_c10_s030_Load',
               'RelnRul',
               'checking existence of parent_comp_type in the CDW CompTyp table before the RelnRul table is updated',
               'child_comp_type_descr: ' || child_comp_type_ds,
               'N/A',
               parent_comp_type,
               'The parent_comp_type, ' || parent_comp_type || ', of class AVAIL_Comp_Type_Relationship for component type, '|| child_comp_type || ', is an undefined component type. No entry for this record can be entered into the RelnRul table.', 
               'Verify that the parent_comp_type is correct and, if it is, send in an event of class AVAIL_Comp_Type_Relationship to define that component type and then send in this failed event again to define the parent/child conponent type relationship this event was trying to define. If the parent component type is not correct resend in an AVAIL_Comp_Type_Relationship for this child component type and description with a corrected parent component type. This event will be deleted.',
               '9999-01-01-00.00.00.000000'
	from EC1.Stage_Comp_Typ_Rel
		where not exists (select CompTyp_Cd from TWG.Cur_CompTyp 
					where CompTyp_Cd = parent_comp_type)
			and parent_comp_type <> 'NULL';
					
-- ********************************************************************************
-- Delete any records whose parent_comp_type doesn't exist in the CompTyp table
-- to avoid referential integrity errors in the warehouse insert in RelnRul table.
-- ********************************************************************************
--#EXECUTE_AT_TARGET
delete from EC1.Stage_Comp_Typ_Rel
	where not exists (select CompTyp_Cd from TWG.Cur_CompTyp 
				where CompTyp_Cd = parent_comp_type)
		and parent_comp_type <> 'NULL';

-- ***************************************************************************
-- Insert any new Component Type values into the TWG.RelnRul table
-- *****************************************************************************
--#EXECUTE_AT_TARGET
declare global temporary table RelnRul (
       CompTyp_Source_Cd    CHAR(17) NOT NULL,
       CompTyp_Target_Cd    CHAR(17) NOT NULL,
       RelnTyp_Cd           CHAR(6) NOT NULL,
       RelnRul_Strt_DtTm    TIMESTAMP NOT NULL,
       RelnRul_End_DtTm     TIMESTAMP NOT NULL)
with replace on commit preserve rows not logged ;

-- There's no relationship to insert if comp_type_parent is the string 'NULL'
--#EXECUTE_AT_TARGET
insert into session.RelnRul 
	select parent_comp_type,
	       child_comp_type,
	       'PCHILD',
	       current timestamp - current timezone, 
	       '9999-01-01-00.00.00.000000'
	from EC1.Stage_Comp_Typ_Rel
		where parent_comp_type <> 'NULL';

--#EXECUTE_AT_TARGET
insert into TWG.RelnRul
	(CompTyp_Source_Cd, CompTyp_Target_Cd, RelnTyp_Cd, RelnRul_Strt_DtTm, RelnRul_End_DtTm)
	select * from session.RelnRul s
		where not exists (select 1 from TWG.RelnRul t 
					where t.CompTyp_Source_Cd = s.CompTyp_Source_Cd and
					      t.CompTyp_Target_Cd = s.CompTyp_Target_Cd and
					      t.RelnTyp_Cd        = s.RelnTyp_Cd); 

-- *****************************************************************************************
-- ****** END: Process the Component Type data into the CompTyp and RelnRul tables
-- *****************************************************************************************

-- *****************************************************************************************
-- ****** START: Process the Component Type State data into the MsmtRul table
-- *****************************************************************************************

-- *************************************************************************************
-- Two types of duplicates are checked for in this step. The first checks for 
-- duplicates that exist within the EC1.Stage_Comp_Rel table, which contains the 
-- current set of extracted events to define new components. And the second
-- checks for duplicates of records in the EC1.Stage_Comp_Rel table that already 
-- exist in the TWG.Comp table. This second way is done with the 'where not exists'
-- clause on the 'insert into TWG.Comp' statement.
-- *************************************************************************************

-- **********************************************************************************
-- Write out an error message for any records with duplicate child_comp and 
-- child_comp_type values. For this ETL we consider a component unique when these
-- two values form a unique pair.
-- **********************************************************************************
--#EXECUTE_AT_TARGET
insert into EC1.Exception_Log
        select Current Timestamp - Current Timezone,
               'EC1_c10_ETL1_Process',
               'EC1_c10_s030_Load',
               'Comp',
               'checking for duplicate component values in the EC1.Stage_Comp_Rel table before TWG.Comp table is updated',
               'child_comp_descr: ' || d1.child_comp_descr,
               'child_comp_type: ' || d1.child_comp_type,
               'child_comp: ' || d1.child_comp,
               'The child_comp, ' || d1.child_comp || ', of class AVAIL_Comp_Relationship is a duplicate component value for component type ' || d1.child_comp_type || '. No entry for this record can be entered into the Comp table. The first event with this component value will be entered but this event will be discarded.', 
               'No recovery is needed. You cannot have identical components of the same component type in the TWG.Comp table. There were duplicate components in this ETLs extracted events. Only the first will be inserted into the TWG.Comp table. All other duplicates of this component and component type combination found in the extracted events will generate an error message and then be deleted.',
               '9999-01-01-00.00.00.000000'
	from EC1.Stage_Comp_Rel d1
		where d1.date_reception > (select min(d2.date_reception) from EC1.Stage_Comp_Rel d2
						where d1.child_comp_type   = d2.child_comp_type and 
						      d1.child_comp        = d2.child_comp and 
               		      		      	      d1.date_reception    > d2.date_reception)
	       	       OR d1.event_hndl > (select min(d2.event_hndl) from EC1.Stage_Comp_Rel d2
						where d1.child_comp_type   = d2.child_comp_type and 
						      d1.child_comp        = d2.child_comp and 
               		      		              d1.date_reception    = d2.date_reception and
               		      		              d1.event_hndl        > d2.event_hndl);
-- *******************************************************************
-- Remove any duplicate events in the newly extracted set of events
-- by removing the duplicates that arrived last.
-- *******************************************************************
--#EXECUTE_AT_TARGET
delete from EC1.Stage_Comp_Rel d1
	where d1.date_reception > (select min(d2.date_reception) from EC1.Stage_Comp_Rel d2
					where d1.child_comp_type   = d2.child_comp_type and 
					      d1.child_comp        = d2.child_comp and 
               		      		      d1.date_reception    > d2.date_reception)
	      OR d1.event_hndl > (select min(d2.event_hndl) from EC1.Stage_Comp_Rel d2
					where d1.child_comp_type   = d2.child_comp_type and 
					      d1.child_comp        = d2.child_comp and 
               		      		      d1.date_reception    = d2.date_reception and
               		      		      d1.event_hndl        > d2.event_hndl);

-- ***********************************************************************
-- Find any events that have invalid states to assign to component type
-- and generate an error message. They are not deleted because the where
-- clause on the 'insert into session.MsmtRul' doesn't allow them to be
-- inserted into the session.MsmtRul table.
-- ***********************************************************************
--#EXECUTE_AT_TARGET
insert into EC1.Exception_Log
        select Current Timestamp - Current Timezone,
               'EC1_c10_ETL1_Process',
               'EC1_c10_s030_Load',
               'MsmtRul',
               'checking validity of the component state with available states in the MsmtTyp table',
               'N/A',
               'N/A',
               state,
               'The measurement type, ' || state || ', to be inserted into the MsmtRul table for component type, '|| comp_type || ', is an invalid measurement type. No entry for this record will be entered into the MsmtRul table.', 
               'Check the valid values in the MsmtTyp table and resend the same AVAIL_Comp_Type_State event through again.', 
               '9999-01-01-00.00.00.000000'
	from EC1.Stage_Comp_Typ_St
		where state NOT in (select MsmtTyp_Nm from TWG.MsmtTyp);

-- ***************************************************************************
-- Write out an error message for any records whose comp_type doesn't exist
-- ***************************************************************************
--#EXECUTE_AT_TARGET
insert into EC1.Exception_Log
        select Current Timestamp - Current Timezone,
               'EC1_c10_ETL1_Process',
               'EC1_c10_s030_Load',
               'MsmtRul',
               'checking existence of comp_type in the CDW CompTyp table before the MsmtRul table is updated',
               'N/A',
               'N/A',
               comp_type,
               'The comp_type, ' || comp_type || ', of class AVAIL_Comp_Type_State for state, '|| state || ', is an undefined component type. No entry for this record can be entered into the MsmtRul table.', 
               'Verify that the comp_type is correct and, if it is, send in events of classes AVAIL_Comp_Type_Relationship and AVAIL_Comp_Type_State to define this component type. If it is not correct send in another event of class AVAIL_Comp_Type_State to correct this invalid event.',
               '9999-01-01-00.00.00.000000'
	from EC1.Stage_Comp_Typ_St
		where not exists (select CompTyp_Cd from TWG.Cur_CompTyp 
					where CompTyp_Cd = comp_type);
					
-- ********************************************************************************
-- Delete any records whose comp_type doesn't exist in the CompTyp table to avoid
-- referential integrity errors in the warehouse insert into the RelnRul table.
-- ********************************************************************************
--#EXECUTE_AT_TARGET
delete from EC1.Stage_Comp_Typ_St
	where not exists (select CompTyp_Cd from TWG.Cur_CompTyp 
				where CompTyp_Cd = comp_type);

-- **********************************************************************************
-- Write out an error message for any records with duplicate comp_type and 
-- state values. 
-- **********************************************************************************
--#EXECUTE_AT_TARGET
insert into EC1.Exception_Log
        select Current Timestamp - Current Timezone,
               'EC1_c10_ETL1_Process',
               'EC1_c10_s030_Load',
               'MsmtRul',
               'checking for duplicate component type states in the EC1.Stage_Comp_Typ_St table before TWG.MsmtRul table is updated',
               'comp_type: ' || d1.comp_type,
               'N/A',
               state,
               'The state, ' || d1.state || ', of class AVAIL_Comp_Typ_State is a duplicate state value for component type ' || d1.comp_type || '. No entry for this record can be entered into the MsmtRul table. The first event with this component type state value will be entered but this event will be discarded.', 
               'No recovery is needed. You cannot have duplicate component type states in the TWG.MsmtRul table. There were duplicate component state definitions in this ETLs extracted events. Only the first will be inserted into the TWG.MsmtRul table. All other duplicates of this component type and state combination found in the extracted events will generate an error message and then be deleted.',
               '9999-01-01-00.00.00.000000'
	from EC1.Stage_Comp_Typ_St d1
		where d1.date_reception > (select min(d2.date_reception) from EC1.Stage_Comp_Typ_St d2
						where d1.comp_type         = d2.comp_type and 
						      d1.state             = d2.state and 
               		      		      	      d1.date_reception    > d2.date_reception)
	       	       OR d1.event_hndl > (select min(d2.event_hndl) from EC1.Stage_Comp_Typ_St d2
						where d1.comp_type         = d2.comp_type and 
						      d1.state             = d2.state and 
               		      		              d1.date_reception    = d2.date_reception and
               		      		              d1.event_hndl        > d2.event_hndl);

-- *******************************************************************
-- Remove any duplicate events in the newly extracted set of events
-- by removing the duplicates that arrived last.
-- *******************************************************************
--#EXECUTE_AT_TARGET
delete from EC1.Stage_Comp_Typ_St d1
	where d1.date_reception > (select min(d2.date_reception) from EC1.Stage_Comp_Typ_St d2
					where d1.comp_type         = d2.comp_type and 
					      d1.state             = d2.state and 
               		      		      d1.date_reception    > d2.date_reception)
	      OR d1.event_hndl > (select min(d2.event_hndl) from EC1.Stage_Comp_Typ_St d2
					where d1.comp_type         = d2.comp_type and 
					      d1.state             = d2.state and 
               		      		      d1.date_reception    = d2.date_reception and
               		      		      d1.event_hndl        > d2.event_hndl);

-- ********************************************************************************
-- Insert the remaining valid component relationships into the CDW MsmtRul table
-- ********************************************************************************
--#EXECUTE_AT_TARGET
declare global temporary table MsmtRul (
  CompTyp_Cd           CHAR(17) NOT NULL,
  MsmtTyp_ID           INTEGER NOT NULL
)
with replace on commit preserve rows not logged;

--#EXECUTE_AT_TARGET
insert into session.MsmtRul
	select comp_type,
	       (select MsmtTyp_ID from TWG.MsmtTyp where MsmtTyp_Nm = state)
	from EC1.Stage_Comp_Typ_St
		where state in (select MsmtTyp_Nm from TWG.MsmtTyp);

-- Insert measurement rules for component types
--#EXECUTE_AT_TARGET
insert into TWG.MsmtRul
	(CompTyp_Cd, MsmtTyp_ID)
	select * from session.MsmtRul s
		where not exists (select 1 from TWG.MsmtRul t 
					where t.CompTyp_Cd = s.CompTyp_Cd and
					      t.MsmtTyp_ID = s.MsmtTyp_ID);

-- *****************************************************************************************
-- ****** END: Process the Component Type State data into the MsmtRul table
-- *****************************************************************************************


-- *****************************************************************************************
-- ****** START: Process the Component data into the Comp and CompReln tables
-- *****************************************************************************************
-- *********************************************************************************
-- Write out an error message for any records whose child_comp_type doesn't exist.
-- *********************************************************************************
--#EXECUTE_AT_TARGET
insert into EC1.Exception_Log
        select Current Timestamp - Current Timezone,
               'EC1_c10_ETL1_Process',
               'EC1_c10_s030_Load',
               'Comp',
               'checking existence of child_comp_type in the CDW CompTyp table before the Comp table is updated',
               'parent_comp_type: ' || parent_comp_type,
               'parent_comp: ' || parent_comp,
               child_comp_type,
               'The child_comp_type, ' || child_comp_type || ', of class AVAIL_Comp_Relationship for component, '|| child_comp || ', is an undefined component type. No entry for this record can be entered into the Comp or CompReln tables.', 
               'Verify that the child_comp_type is correct and, if it is, send in events of classes AVAIL_Comp_Type_Relationship and AVAIL_Comp_Type_State to define this undefined component type. If it is not correct send in another event of class AVAIL_Comp_Relationship to correct this invalid event. This event will be discarded.',
               '9999-01-01-00.00.00.000000'
	from EC1.Stage_Comp_Rel
		where not exists (select CompTyp_Cd from TWG.Cur_CompTyp 
					where CompTyp_Cd = child_comp_type);
					
-- ********************************************************************************
-- Delete any records whose child_comp_type doesn't exist in the CompTyp table
-- to avoid referential integrity errors in the warehouse insert into the 
-- Comp and CompReln tables.  
-- ********************************************************************************
--#EXECUTE_AT_TARGET
delete from EC1.Stage_Comp_Rel
	where not exists (select CompTyp_Cd from TWG.Cur_CompTyp 
				where CompTyp_Cd = child_comp_type);

-- *********************************************************************************
-- Write out an error message for any records whose parent_comp_type doesn't exist.
-- *********************************************************************************
--#EXECUTE_AT_TARGET
insert into EC1.Exception_Log
        select Current Timestamp - Current Timezone,
               'EC1_c10_ETL1_Process',
               'EC1_c10_s030_Load',
               'Comp',
               'checking existence of parent_comp_type in the CDW CompTyp table before the Comp table is updated',
               'child_comp_type: ' || child_comp_type,
               'child_comp: ' || child_comp,
               parent_comp_type,
               'The parent_comp_type, ' || parent_comp_type || ', of class AVAIL_Comp_Relationship for parent component, '|| parent_comp || ', is an undefined component type. No entry for this record can be entered into the Comp or CompReln table.', 
               'Verify that the parent_comp_type is correct and, if it is, send in events of classes AVAIL_Comp_Type_Relationship and AVAIL_Comp_Type_State to define this undefined component type. If it is not correct send in another event of class AVAIL_Comp_Relationship to correct this invalid event. This event will be discarded.',
               '9999-01-01-00.00.00.000000'
	from EC1.Stage_Comp_Rel
		where parent_comp_type <> 'NULL'
			and not exists (select CompTyp_Cd from TWG.Cur_CompTyp 
					where CompTyp_Cd = parent_comp_type);
					
-- ********************************************************************************
-- Delete any records whose parent_comp_type doesn't exist in the CompTyp table
-- to avoid referential integrity errors in the warehouse insert into the 
-- Comp and CompReln tables. Leave events in for now whose parent_comp_type 
-- is 'NULL' until after they have been inserted into the Comp table where
-- the parent values are not needed.
-- ********************************************************************************
--#EXECUTE_AT_TARGET
delete from EC1.Stage_Comp_Rel
	where parent_comp_type <> 'NULL'
		and not exists (select CompTyp_Cd from TWG.Cur_CompTyp 
				where CompTyp_Cd = parent_comp_type);

-- ***********************************************************************************
-- Write out an error message for any records whose parent_comp doesn't exist.
-- Since the parent_comp could be in the current set of events to be loaded into the
-- Comp table too we must also check the EC1.Stage_Comp_Rel table to see if the 
-- parent_comp is defined in this same batch. The parent_comp must be defined BEFORE
-- the child_comp so the date_reception and event_hndl values are checked to ensure
-- the correct order, otherwise it's an error and the event will be dropped.
-- ***********************************************************************************
--#EXECUTE_AT_TARGET
insert into EC1.Exception_Log
        select Current Timestamp - Current Timezone,
               'EC1_c10_ETL1_Process',
               'EC1_c10_s030_Load',
               'Comp',
               'checking existence of parent_comp in the CDW Comp table before the Comp table is updated with the child component',
               'parent_comp_type: ' || parent_comp_type,
               'child_comp: ' || child_comp,
               parent_comp,
               'The parent_comp, ' || parent_comp || ', of class AVAIL_Comp_Relationship for component, '|| child_comp || ', is an undefined component. No entry for this record can be entered into the Comp or CompReln tables.', 
               'Verify that the parent_comp is correct and, if it is, send in events of class AVAIL_Comp_Relationship to define this undefined component. If it is not correct send in another event of class AVAIL_Comp_Relationship with the correct parent_comp value to correct this invalid event. This event will be discarded.',
               '9999-01-01-00.00.00.000000'
	from EC1.Stage_Comp_Rel cr1
		where parent_comp <> 'NULL'
		and (not exists (select Comp_Nm from TWG.Cur_Comp 
				where Comp_Nm = parent_comp 
					and CompTyp_Cd = parent_comp_type
					and parent_comp_type <> 'NULL')
			and not exists (select cr2.child_comp from EC1.Stage_Comp_Rel cr2
					where cr2.child_comp = cr1.parent_comp
						and cr2.child_comp_type = cr1.parent_comp_type
						and cr2.date_reception = cr1.date_reception
						and cr2.event_hndl < cr1.event_hndl)
			and not exists (select cr2.child_comp from EC1.Stage_Comp_Rel cr2
					where cr2.child_comp = cr1.parent_comp
						and cr2.child_comp_type = cr1.parent_comp_type
						and cr2.date_reception < cr1.date_reception));

-- ********************************************************************************
-- Delete any records whose parent_comp doesn't exist in the Comp table
-- or whose parent_comp is defined after a child record references it.
-- This is to avoid referential integrity errors in the warehouse insert into the 
-- Comp and CompReln tables. 
-- ********************************************************************************
--#EXECUTE_AT_TARGET
delete from EC1.Stage_Comp_Rel cr1
	where parent_comp <> 'NULL' 
		and (not exists (select Comp_Nm from TWG.Cur_Comp 
				where Comp_Nm = parent_comp 
					and CompTyp_Cd = parent_comp_type)
			and not exists (select cr2.child_comp from EC1.Stage_Comp_Rel cr2
				where cr2.child_comp = cr1.parent_comp
					and cr2.child_comp_type = cr1.parent_comp_type
					and cr2.date_reception = cr1.date_reception
					and cr2.event_hndl < cr1.event_hndl)
			and not exists (select cr2.child_comp from EC1.Stage_Comp_Rel cr2
				where cr2.child_comp = cr1.parent_comp
					and cr2.child_comp_type = cr1.parent_comp_type
					and cr2.date_reception < cr1.date_reception));

-- ********************************************************************************
-- Insert the remaining valid components into the CDW Comp table
-- ********************************************************************************
--#EXECUTE_AT_TARGET
insert into TWG.Comp
	(Comp_ID, CompTyp_Cd, Centr_Cd, Cust_ID, Comp_Nm, Comp_Strt_DtTm, Comp_End_DtTm, Comp_Ds)
        select 0 				    as Comp_Id,
	       co.child_comp_type 		    as CompTyp_Cd,
	       ct.Centr_Cd 			    as Centr_Cd,
	       cu.Cust_Id 			    as Cust_Id,
	       co.child_comp 			    as Comp_Nm,
               current timestamp - current timezone as Comp_Strt_DtTm,
               '9999-01-01-00.00.00.000000'         as Comp_End_DtTm,
	       case
		   when co.child_comp_descr = 'NULL' then ' '
		   else co.child_comp_descr
	       end as Comp_Ds
	from EC1.Stage_Comp_Rel co, EC1.Cust_Lookup cu, EC1.Centr_Lookup ct
        	where not exists (select 1 from TWG.Cur_Comp t
					where t.CompTyp_Cd = co.child_comp_type and
					      t.Comp_Nm    = co.child_comp)
			and cu.Value = '@' and ct.Value = '@'; 

-- *************************************************************************
-- Remove any rows that have NULL parent_comp or parent_comp_type's because
-- no component relationship can be made if the parent values are NULL.
-- *************************************************************************
--#EXECUTE_AT_TARGET
delete from EC1.Stage_Comp_Rel
	where parent_comp_type = 'NULL' or parent_comp = 'NULL';

-- *************************************************************************
-- Populate the CDW CompReln table with new Component Relationship values
-- *************************************************************************
--#EXECUTE_AT_TARGET
declare global temporary table CompReln (
       Comp_Source_ID       INTEGER NOT NULL,
       Comp_Target_ID       INTEGER NOT NULL,
       RelnTyp_Cd           CHAR(6) NOT NULL
) 
with replace on commit preserve rows not logged ;

-- Insert component relationship values only if the parent values exist in TWG.Comp
--#EXECUTE_AT_TARGET
insert into session.CompReln
	select (select Comp_ID from TWG.Cur_Comp 
			where parent_comp_type = CompTyp_Cd and parent_comp = Comp_Nm),
	       (select Comp_ID from TWG.Cur_Comp 
			where child_comp_type = CompTyp_Cd and child_comp = Comp_Nm),
               'PCHILD' 
	from EC1.Stage_Comp_Rel
		where exists (select Comp_ID from TWG.Cur_Comp 
			where parent_comp_type = CompTyp_Cd and parent_comp = Comp_Nm);

--#EXECUTE_AT_TARGET
insert into TWG.CompReln
        select 0                                    as CompReln_Id,
               c.Comp_Source_ID                     as Comp_Source_ID,
               c.Comp_Target_ID                     as Comp_Target_ID,
               c.RelnTyp_Cd                         as RelnTyp_Cd,
               current timestamp - current timezone as CompReln_Strt_DtTm,
               '9999-01-01-00.00.00.000000'         as CompReln_End_DtTm
        from session.CompReln c 
        	where not exists
                	(select 1 from TWG.Cur_CompReln r
                        	where r.Comp_Source_ID = c.Comp_Source_ID and
                                      r.Comp_Target_ID = c.Comp_Target_ID and
                          	      r.RelnTyp_Cd     = c.RelnTyp_Cd);


-- *****************************************************************************************
-- ****** END: Process the Component data into the Comp and CompReln tables
-- *****************************************************************************************

-- *************************************************************************************
-- ****** START: Process the measurement group data into the MGrp table
-- *************************************************************************************

-- *************************************************************************************
-- Two types of duplicates are checked for in this step. The first checks for 
-- duplicates that exist within the EC1.Stage_Msr_Gp_Typ table, which contains the 
-- current set of extracted events to define new measurement groups. And the second
-- checks for duplicates of records in the EC1.Stage_Msr_Gp_Typ table that already 
-- exist in the TWG.MGrp table. This second way is done with the 'where not exists'
-- clause on the 'insert into TWG.MGrp' statement.
-- *************************************************************************************

-- **********************************************************************************
-- Write out an error message for any records with duplicate measurement code values
-- **********************************************************************************
--#EXECUTE_AT_TARGET
insert into EC1.Exception_Log
        select Current Timestamp - Current Timezone,
               'EC1_c10_ETL1_Process',
               'EC1_c10_s030_Load',
               'MGrp',
               'checking for duplicate measurement code values in the EC1.Stage_Msr_Gp_Typ table before TWG.MGrp table is updated',
               'descr: ' || descr,
               'N/A',
               'code: ' || d1.code,
               'The measurement group code, ' || d1.code || ', of class AVAIL_Measurement_Group_Type is a duplicate measurement group code. No entry for this record can be entered into the MGrp table. The first event with this measurement group code will be entered but this event will be discarded.', 
               'No recovery is needed. You cannot have identical measurement group codes in the TWG.MGrp table. There were duplicate measurement group codes in this ETLs extracted events. Only the first will be inserted into the TWG.MGrp table. All other duplicates of this measurement group code in the extracted events will generate an error message and then be deleted.',
               '9999-01-01-00.00.00.000000'
	from EC1.Stage_Msr_Gp_Typ d1
		where d1.date_reception > (select min(d2.date_reception) from EC1.Stage_Msr_Gp_Typ d2
						where d1.code              = d2.code and 
               		      		      	      d1.date_reception    > d2.date_reception)
	       	       OR d1.event_hndl > (select min(d2.event_hndl) from EC1.Stage_Msr_Gp_Typ d2
						where d1.code              = d2.code and 
               		      		              d1.date_reception    = d2.date_reception and
               		      		              d1.event_hndl        > d2.event_hndl);
-- *******************************************************************
-- Remove any duplicate events in the newly extracted set of events
-- by removing the duplicates that arrived last.
-- *******************************************************************
--#EXECUTE_AT_TARGET
delete from EC1.Stage_Msr_Gp_Typ d1
		where d1.date_reception > (select min(d2.date_reception) from EC1.Stage_Msr_Gp_Typ d2
						where d1.code              = d2.code and 
               		      		      	      d1.date_reception    > d2.date_reception)
	       	       OR d1.event_hndl > (select min(d2.event_hndl) from EC1.Stage_Msr_Gp_Typ d2
						where d1.code              = d2.code and 
               		      		              d1.date_reception    = d2.date_reception and
               		      		              d1.event_hndl        > d2.event_hndl);

-- *************************************************************************
-- Populate the CDW MGrp table with new measurement group values
-- *************************************************************************
--#EXECUTE_AT_TARGET
insert into TWG.MGrp
	(MGrp_Cd, MGrpTyp_Cd, MGrp_Nm)
	select code    as MGrp_Cd,
	       'TRANS' as MGrpTyp_Cd,
	       descr   as MGrp_Nm
	from EC1.Stage_Msr_Gp_Typ
		where not exists (select 1 from TWG.MGrp t 
					where t.mgrp_Cd = code);

-- *************************************************************************************
-- ****** END: Process the measurement group data into the MGrp table
-- *************************************************************************************



-- *************************************************************************************
-- ****** START: Process the measurement group state data into the MGrpMbr table
-- *************************************************************************************

-- ***************************************************************************************
-- Write out an error message for any records whose measurement group code doesn't exist.
-- ***************************************************************************************
--#EXECUTE_AT_TARGET
insert into EC1.Exception_Log
        select Current Timestamp - Current Timezone,
               'EC1_c10_ETL1_Process',
               'EC1_c10_s030_Load',
               'MGrpMbr',
               'checking existence of measurement group code in the CDW MGrp table before the MGrpMbr table is updated',
               'N/A',
               'N/A',
               code,
               'The measurement group code, ' || code || ', of class AVAIL_Measurement_Group_State_Type for state, '|| state || ', is an undefined measurement group code. No entry for this record can be entered into the MGrpMbr table.', 
               'Verify that the measurement group code is correct and, if it is, send in an event of class AVAIL_Measurement_Group_Type to define this undefined measurement group code. If it is not correct send in another event of class AVAIL_Measurement_Group_State_Type to correct this invalid event. This event will be discarded.',
               '9999-01-01-00.00.00.000000'
	from EC1.Stage_Msr_Gp_State
		where not exists (select MGrp_Cd from TWG.MGrp
					where MGrp_Cd = code);
					
-- ********************************************************************************
-- Delete any records whose measurement group code doesn't exist in the MGrp table
-- to avoid referential integrity errors in the warehouse insert in MGrpMbr table.
-- ********************************************************************************
--#EXECUTE_AT_TARGET
delete from EC1.Stage_Msr_Gp_State
	where not exists (select MGrp_Cd from TWG.MGrp
				where MGrp_Cd = code);

-- ***********************************************************************************
-- Write out an error message for any records whose measurement type doesn't exist.
-- They are not deleted because the where clause on the 'insert into session.MGrpMbr'
-- doesn't allow them to be inserted into the session.MGrpMbr table.
-- ***********************************************************************************
-- Check for invalid measurement types in the code slot
--#EXECUTE_AT_TARGET
insert into EC1.Exception_Log
        select Current Timestamp - Current Timezone,
               'EC1_c10_ETL1_Process',
               'EC1_c10_s030_Load',
               'MGrpMbr',
               'checking validity of the measurement state with available values in the MsmtTyp table',
               'N/A',
	       'N/A',
	        state,
               'The measurement type, ' || state || ', to be inserted into the MGrbMbr table for measurement group, '|| code || ', is an invalid measurement type. No entry for this record will be entered into the MGrbMbr table.', 
               'Check the valid values in the MsmtTyp table and resend an AVAIL_Measurement_Group_State_Type event through again to correct this invalid event.', 
               '9999-01-01-00.00.00.000000'
        from EC1.Stage_Msr_Gp_State
		where state NOT in (select MsmtTyp_Nm from TWG.MsmtTyp);

-- **********************************************************************************
-- Write out an error message for any records with duplicate measurement group code 
-- and state values. 
-- **********************************************************************************
--#EXECUTE_AT_TARGET
insert into EC1.Exception_Log
        select Current Timestamp - Current Timezone,
               'EC1_c10_ETL1_Process',
               'EC1_c10_s030_Load',
               'MGrpMbr',
               'checking for duplicate measurement group state definitions in the EC1.Stage_Msr_Gp_State table before TWG.MGrpMbr table is updated',
               'measurement group code: ' || d1.code,
               'N/A',
               state,
               'The state, ' || d1.state || ', of class AVAIL_Measurement_Group_State_Type is a duplicate state value for measurement group code ' || d1.code || '. No entry for this record can be entered into the MGrpMbr table. The first event with this state value will be entered but this event will be discarded.', 
               'No recovery is needed. You cannot have duplicate measurement group states in the TWG.MGrpMbr table. There were duplicate measurement group state definitions in this ETLs extracted events. Only the first will be inserted into the TWG.MGrpMbr table. All other duplicates of this measurement group and state combination found in the extracted events will generate an error message and then be deleted.',
               '9999-01-01-00.00.00.000000'
	from EC1.Stage_Msr_Gp_State d1
		where d1.date_reception > (select min(d2.date_reception) from EC1.Stage_Msr_Gp_State d2
						where d1.code              = d2.code and 
						      d1.state             = d2.state and 
               		      		      	      d1.date_reception    > d2.date_reception)
	       	       OR d1.event_hndl > (select min(d2.event_hndl) from EC1.Stage_Msr_Gp_State d2
						where d1.code              = d2.code and 
						      d1.state             = d2.state and 
               		      		              d1.date_reception    = d2.date_reception and
               		      		              d1.event_hndl        > d2.event_hndl);

-- *******************************************************************
-- Remove any duplicate events in the newly extracted set of events
-- by removing the duplicates that arrived last.
-- *******************************************************************
--#EXECUTE_AT_TARGET
delete from EC1.Stage_Msr_Gp_State d1
		where d1.date_reception > (select min(d2.date_reception) from EC1.Stage_Msr_Gp_State d2
						where d1.code              = d2.code and 
						      d1.state             = d2.state and 
               		      		      	      d1.date_reception    > d2.date_reception)
	       	       OR d1.event_hndl > (select min(d2.event_hndl) from EC1.Stage_Msr_Gp_State d2
						where d1.code              = d2.code and 
						      d1.state             = d2.state and 
               		      		              d1.date_reception    = d2.date_reception and
               		      		              d1.event_hndl        > d2.event_hndl);

-- ***************************************************************************************
-- Populate the CDW MGrpMbr table with the remaining measurement group measurement types
-- ***************************************************************************************
-- create a temporary table for measurement group members
--#EXECUTE_AT_TARGET
declare global temporary table MGrpMbr (
        MGrp_Cd              CHAR(6) NOT NULL,
        MGrpTyp_Cd           CHAR(6) NOT NULL,
        MsmtTyp_Id               INTEGER NOT NULL
        )
  with replace on commit preserve rows not logged;

-- insert the measurement group types into the CDW if it doesn't already exist
--#EXECUTE_AT_TARGET
insert into session.MGrpMbr 
	(MGrp_Cd, MGrpTyp_Cd, MsmtTyp_Id)
        select code,
               'TRANS',
               (select MsmtTyp_ID from TWG.MsmtTyp where state = MsmtTyp_Nm)
        from EC1.Stage_Msr_Gp_State
		where state in (select MsmtTyp_Nm from TWG.MsmtTyp);

--#EXECUTE_AT_TARGET
insert into TWG.MGrpMbr 
	(MGrp_Cd, MGrpTyp_Cd, MsmtTyp_Id)
        select s.MGrp_Cd,
               s.MGrpTyp_Cd,
               s.MsmtTyp_Id
        from session.MGrpMbr s
        	where not exists (select 1 from TWG.MGrpMbr t 
					where t.MGrp_Cd    = s.MGrp_Cd and
                			      t.MGrpTyp_Cd = s.MGrpTyp_Cd and
                			      t.MsmtTyp_Id = s.MsmtTyp_Id);

-- *************************************************************************************
-- ****** END: Process the measurement group state data into the MGrpMbr table
-- *************************************************************************************

-- *************************************************************************************
-- ****** START: Process the AVAIL Outage data into the Msmt table
-- *************************************************************************************

-- ***************************************************************************************
-- Write out an error message for any records whose component type doesn't exist.
-- ***************************************************************************************
--#EXECUTE_AT_TARGET
insert into EC1.Exception_Log
        select Current Timestamp - Current Timezone,
               'EC1_c10_ETL1_Process',
               'EC1_c10_s030_Load',
               'Msmt',
               'checking existence of component type for outage event in the CDW CompTyp table before the Msmt table is updated',
               'outage start time: ' || outage_start_time || 'outage_duration: ' || char(outage_duration),
               'state_became: ' || state_became,
               comp_type,
               'The component type, ' || comp_type || ', of class AVAIL_Outage for component, '|| comp_instance || ', is an undefined component type. No entry for this record can be entered into the Msmt table.', 
               'Verify that the comp_type is correct and, if it is, send in events of classes AVAIL_Comp_Type_Relationship and AVAIL_Comp_Type_State to define this undefined component type. If it is not correct send in another event of class AVAIL_Outage to correct this invalid event. This event will be discarded.',
               '9999-01-01-00.00.00.000000'
	from EC1.Stage_Outage_Data
		where not exists (select CompTyp_Cd from TWG.Cur_CompTyp
					where CompTyp_Cd = comp_type);
					
-- ********************************************************************************
-- Delete any records whose comp_type doesn't exist in the CompTyp table
-- to avoid referential integrity errors in the warehouse insert in Msmt table.
-- ********************************************************************************
--#EXECUTE_AT_TARGET
delete from EC1.Stage_Outage_Data
	where not exists (select CompTyp_Cd from TWG.Cur_CompTyp
				where CompTyp_Cd = comp_type);

-- ***************************************************************************************
-- Write out an error message for any records whose component doesn't exist.
-- ***************************************************************************************
--#EXECUTE_AT_TARGET
insert into EC1.Exception_Log
        select Current Timestamp - Current Timezone,
               'EC1_c10_ETL1_Process',
               'EC1_c10_s030_Load',
               'Msmt',
               'checking existence of the component in the CDW Comp table before the Msmt table is updated',
               'outage start time: ' || outage_start_time || 'outage_duration: ' || char(outage_duration),
               'state_became: ' || state_became,
               comp_instance,
               'The component, ' || comp_instance || ', of class AVAIL_Outage for component type, '|| comp_type || ', is an undefined component. No entry for this record can be entered into the Msmt table.', 
               'Verify that the component is correct and, if it is, send in an event of class AVAIL_Comp_Relationship to define this undefined component. If it is not correct it and send in another event of class AVAIL_Outage to correct this invalid event. This event will be discarded.',
               '9999-01-01-00.00.00.000000'
	from EC1.Stage_Outage_Data
		where not exists (select Comp_Nm from TWG.Cur_Comp
					where CompTyp_Cd = comp_type
						and Comp_Nm = comp_instance);
					
-- ********************************************************************************
-- Delete any records whose component doesn't exist in the Comp table
-- to avoid referential integrity errors in the warehouse insert in Msmt table.
-- ********************************************************************************
--#EXECUTE_AT_TARGET
delete from EC1.Stage_Outage_Data
	where not exists (select Comp_Nm from TWG.Cur_Comp
				where CompTyp_Cd = comp_type
					and Comp_Nm = comp_instance);

-- Check for invalid measurement types in the state_became slot
-- ***************************************************************************************
-- Write out an error message for any records whose measurement type in the state_became
-- slot is invalid.
-- ***************************************************************************************
--#EXECUTE_AT_TARGET
insert into EC1.Exception_Log
        select Current Timestamp - Current Timezone,
               'EC1_c10_ETL1_Process',
               'EC1_c10_s030_Load',
               'Msmt',
               'checking validity of measurement type in state_became slot with available values in the MsmtTyp and MsmtRul tables',
               'N/A',
               'N/A',
               state_became,
               'The measurement type, ' || state_became || ', for the AVAIL_outage event with component ' || comp_instance || ' has an invalid measurement type. No entry for this record will be entered into the Msmt table.', 
               'Check the valid values in the CDW MsmtTyp and MsmtRul tables and resend an AVAIL_Outage event through again with a valid state_became slot value.', 
               '9999-01-01-00.00.00.000000'
        from EC1.Stage_Outage_Data
		where state_became NOT in (select MsmtTyp_Nm from TWG.MsmtTyp
						where MSrc_Cd = 'MODEL1' and MUnit_Cd = 'Min');

-- **********************************************************
-- Drop the events that had invalid measurement states
-- **********************************************************
--#EXECUTE_AT_TARGET
delete from EC1.Stage_Outage_Data
	where state_became NOT in (select MsmtTyp_Nm from TWG.MsmtTyp
					where MSrc_Cd = 'MODEL1' and MUnit_Cd = 'Min');

-- **************************************************************************************
-- Two types of duplicates are checked for in this step. Duplicates that exist within 
-- the EC1.Stage_Outage_Data table, which contains the current set of events extracted, 
-- and duplicates of records in the EC1.Stage_Outage_Data that already exist in the 
-- TWG.Msmt table.
-- **************************************************************************************
-- *******************************************************************
-- Remove any duplicate events in the newly extracted set of events
-- by removing the duplicates that arrived last.
-- *******************************************************************
--#EXECUTE_AT_TARGET
delete from EC1.Stage_Outage_Data d1
	where d1.date_reception > (select d2.date_reception from EC1.Stage_Outage_Data d2
					where d1.comp_type         = d2.comp_type and 
					      d1.comp_instance     = d2.comp_instance and 
					      d1.state_became      = d2.state_became and 
               		      		      d1.outage_start_time = d2.outage_start_time and
               		      		      d1.outage_duration   = d2.outage_duration and
               		      		      d1.date_reception    > d2.date_reception) 
	      OR d1.event_hndl > (select d2.event_hndl from EC1.Stage_Outage_Data d2
					where d1.comp_type         = d2.comp_type and 
					      d1.comp_instance     = d2.comp_instance and 
					      d1.state_became      = d2.state_became and 
               		      		      d1.outage_start_time = d2.outage_start_time and
               		      		      d1.outage_duration   = d2.outage_duration and
               		      		      d1.date_reception    = d2.date_reception and
               		      		      d1.event_hndl        > d2.event_hndl);

-- ************************************************************
-- Prepare to populate the CDW Msmt table with new outage data
-- ************************************************************
-- Create a temporary table in which to place the new outage events from this extract run
--#EXECUTE_AT_TARGET
declare global temporary table Msmt(
       Comp_ID              INTEGER NOT NULL,
       MsmtTyp_ID           INTEGER NOT NULL,
       TmSum_Cd             CHAR NOT NULL,
       Msmt_Strt_Dt         DATE NOT NULL,
       Msmt_Strt_Tm         TIME NOT NULL,
       Msmt_Tot_Val         FLOAT 
)
  with replace on commit preserve rows not logged;

-- Create a temporary table to hold a copy of the Msmt table for comparison purposes.
-- This is an ETL requirement because doing an insert into the Msmt table using 
-- 'where not exists' clause causes performance problems on the Msmt table.
--#EXECUTE_AT_TARGET
declare global temporary table temp_Msmt (
       Comp_ID              INTEGER NOT NULL,
       MsmtTyp_ID           INTEGER NOT NULL,
       TmSum_Cd             CHAR NOT NULL,
       Msmt_Strt_Dt         DATE NOT NULL,
       Msmt_Strt_Tm         TIME NOT NULL,
       Msmt_Tot_Val         FLOAT
)
  with replace on commit preserve rows not logged;

-- Insert the outage values into the temporary Msmt table.
-- APAR IY53030 Duration time, which is in seconds, is now divided by 60 to give the time in 
-- minutes. 
--#EXECUTE_AT_TARGET 
insert into session.Msmt
        select (select Comp_ID from TWG.Cur_Comp where comp_type = CompTyp_Cd and
					           comp_instance = Comp_Nm),
               (select MsmtTyp_ID from TWG.MsmtTyp where state_became = MsmtTyp_Nm and
					           MSrc_Cd = 'MODEL1' and MUnit_Cd = 'Min'),
               'P',
               date(outage_start_time),
               time(outage_start_time),
               outage_duration / 60.0
        from EC1.Stage_Outage_Data
		where state_became in (select MsmtTyp_Nm from TWG.MsmtTyp
						where MSrc_Cd = 'MODEL1' and MUnit_Cd = 'Min');

-- Insert Msmt table records in a temporary table so the next step can
-- scan it for any duplicates. We narrow what is returned by getting
-- only Msmt records that have a MsmtTyp_ID with an MSrc_Cd of 'MODEL1'
-- and a MUnint_Cd of 'Min'. These are the only measurement values that 
-- an Availability event can have.
--#EXECUTE_AT_TARGET
insert into session.temp_Msmt
	( Comp_ID, MsmtTyp_ID, TmSum_Cd, Msmt_Strt_Dt, Msmt_Strt_Tm, Msmt_Tot_Val)
	select Comp_ID, MsmtTyp_ID, TmSum_Cd, Msmt_Strt_Dt, Msmt_Strt_Tm, Msmt_Tot_Val
        	from TWG.Msmt 
        		where MsmtTyp_ID IN 
				(select MsmtTyp_ID from TWG.MsmtTyp 
					where MSrc_Cd = 'MODEL1'
						and MUnit_Cd = 'Min');

-- Delete any records that are dups of an already existing Msmt table record.
--#EXECUTE_AT_TARGET
delete from session.Msmt m1
        	where exists (select 1 from session.temp_Msmt m2
					where m2.Comp_ID      = m1.Comp_ID and
                		              m2.MsmtTyp_ID   = m1.MsmtTyp_ID and
                			      m2.TmSum_Cd     = m1.TmSum_Cd and
                			      m2.Msmt_Strt_Dt = m1.Msmt_Strt_Dt and
                			      m2.Msmt_Strt_Tm = m1.Msmt_Strt_Tm and
                			      m2.Msmt_Tot_Val = m1.Msmt_Tot_Val);

-- Insert the outage data into the CDW Msmt Table 
--#EXECUTE_AT_TARGET
insert into TWG.Msmt
	(Msmt_ID, Comp_ID, MsmtTyp_ID, TmSum_Cd, Msmt_Strt_Dt, Msmt_Strt_Tm, Msmt_Tot_Val)
	select 0            as Msmt_ID,
	       Comp_ID      as Comp_ID,
	       MsmtTyp_ID   as MsmtTyp_ID,
	       TmSum_Cd     as TmSum_Cd,
	       Msmt_Strt_Dt as Msmt_Strt_Dt,
	       Msmt_Strt_Tm as Msmt_Strt_Tm,
	       Msmt_Tot_Val as Msmt_Tot_Val
        from session.Msmt;

-- ****************************************************************
-- ****** END: Process the AVAIL Outage data into the Msmt table
-- ****************************************************************

-- **********************************************************************************
-- Write an error message for any component types that do not have measurement
-- states assigned in the MsmtRul table. These must be defined or the ITLSA reports
-- will not work properly.
-- **********************************************************************************
--#EXECUTE_AT_TARGET
insert into EC1.Exception_Log
        select Current Timestamp - Current Timezone,
               'EC1_c10_ETL1_Process',
               'EC1_c10_s030_Load',
               'N/A',
               'checking to ensure that each component type has measurement states assigned in the MsmtRul table',
               'child_comp_type_ds: ' || child_comp_type_ds,
               'parent_comp_type: ' || parent_comp_type,
               'child_comp_type: ' || child_comp_type,
               'The component type, ' || child_comp_type || ', of class AVAIL_Comp_Typ_Rel does not have an entry in the MsmtRul table. This will most likely cause errors when trying to create reports from this data.', 
               'You should define measurement state values for this component type with an event of class AVAIL_Comp_Type_State before attempting to run any warehouse reports. You should also be sure there is a defined measurement group with defined measurement states for this component type. If not, you will need to send in events of classes AVAIL_Measurement_Group_Type and AVAIL_Measurement_Group_State_Type.',
               '9999-01-01-00.00.00.000000'
	from EC1.Stage_Comp_Typ_Rel 
		where child_comp_type NOT IN (select CompTyp_Cd from TWG.MsmtRul);

-- **********************************************************************************
-- Write an error message for any measurement groups that do not have measurement
-- states assigned in the MGrpMbr table. These must be defined or the ITLSA reports
-- will not work properly.
-- **********************************************************************************
--#EXECUTE_AT_TARGET
insert into EC1.Exception_Log
        select Current Timestamp - Current Timezone,
               'EC1_c10_ETL1_Process',
               'EC1_c10_s030_Load',
               'N/A',
               'checking to ensure that each measurement group has measurement states assigned in the MGrpMbr table',
               'descr: ' || descr,
               'N/A',
               'code: ' || code,
               'The measurement group, ' || code || ', of class AVAIL_Msr_Gp_Typ does not have an entry or entries in the MGrpMbr table. This will most likely cause errors when trying to create reports from this data.', 
               'You should define measurement state values for this measurement group with an event of class AVAIL_Measurement_Group_State_Type before attempting to run any warehouse reports.',
               '9999-01-01-00.00.00.000000'
	from EC1.Stage_Msr_Gp_Typ
		where code NOT IN (select MGrp_Cd from TWG.MGrpMbr);

-- ************************************************************
-- ******* START: Print any exceptions generated by the Load
-- ************************************************************
-- 'Error' is used here because the only messages we generate in the load are errors.
--#EXECUTE_AT_TARGET
select '***** ERROR *****',
       concat('Calculation: ',Calculation),
       concat('Error Text: ',Error_Msg_Text),
       concat('For Extract Step: ',Step_Nm),
       concat('For Insert into CDW Table: ',Table_Nm),
       concat('Recovery: ',Error_Recovery),
       concat('Value in Error: ',Actual_Value),
       concat('Other event slot values: ',Result_Value_Low),
       concat('Other event slot values: ',Result_Value_High)
                from EC1.Exception_Log
                        where Reported_DtTm = '9999-01-01-00.00.00.000000';
-- **************************************************************************************
-- ******* END: Print any extract generated exceptions to the log for this process run
-- **************************************************************************************

-- *****************************************************
-- ******* START: Change logged error timpstamps *******
-- *****************************************************
-- Change the timestamps on any logged error or warning 
-- to be the same so they can be tied back to the same 
-- extract process run.
--#EXECUTE_AT_TARGET
update EC1.Exception_Log
        set Reported_DtTm = Current Timestamp - Current Timezone
        where Reported_DtTm = '9999-01-01-00.00.00.000000';

-- *****************************************************
-- *******  END: Change logged error timpstamps  *******
-- *****************************************************

-- ******************************************************************
-- ****** START: Prune any Exception_Log data over a month old ******
-- ******************************************************************
--#EXECUTE_AT_TARGET
delete from EC1.Exception_Log
	where Reported_DtTm < Current Timestamp - 1 month;

-- ******************************************************************
-- ******   END: Prune any Exception_Log data over a month old ******
-- ******************************************************************

